/*

Copyright  2003, Russell C. Hay
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

i. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer. 

ii. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

iii. Neither the name of the dead[protocol]society nor the names of its
     contributors may be used to endorse or promote products derived from this
     software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/
#include <sys/types.h>
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <signal.h>
#include "stl.h"
#include "mtxtype.h"
#include "action.h"
#include "protocol.h"
#include "device.h"
#include "connectionMgr.h"

device *g_device;
void sigHandler(int i);

int main()
{
	protocol prot;

	unlink("/tmp/lcd");

	g_device = new device;
	g_device->Open();
	g_device->Backlight(1);
	g_device->Reset();

	signal(SIGHUP, sigHandler);
	signal(SIGINT, sigHandler);
	signal(SIGQUIT, sigHandler);
	signal(SIGILL, sigHandler);
	signal(SIGABRT, sigHandler);
	signal(SIGFPE, sigHandler);
	signal(SIGKILL, sigHandler);
	signal(SIGBUS, sigHandler);
	signal(SIGSEGV, sigHandler);
	signal(SIGTERM, sigHandler);

	connectionMgr mgr(&prot, g_device);

	if (mgr.InitializeEphemeril("/tmp/lcd")<0)
	{
		g_device->Backlight(0);
		perror("InitEmph");
		return 1;
	}

	mgr.Run();
	g_device->Backlight(0);
}

void sigHandler(int i)
{
	printf("\n\nReceived Signal %d\n", i);
	g_device->Backlight(0);
	delete g_device;
	exit(1);
}
